/* avl.h : implementation structures of an avl tree data structure for
   use with circlemud systems ONLY.

   This program can be used, redistributed or modified under the terms
   listed in the LICENSE file included with this package.  If you did
   not receive such a file, you should request one from the package
   provider.

   This software is provided as is, without any warranty express or
   implied.  Use at your own risk.

   6/6/01 - Original revision - Patrick Dughi (dughi@imaxx.net)
*/

typedef int (avl_compare) (void *left, void *right);
typedef void (avl_clear) (void *data);

struct avl_tree {
  struct real_avl_tree *avl;
  avl_compare *compare;
  void *retval;
  int count;
};


struct real_avl_tree {
  struct real_avl_tree *left;
  struct real_avl_tree *right;
  int count;  
  void *value;
  int balance;
};

/* wasteful, perhaps, but it is just a linked list of pointers.. */

struct avl_tree_traverse {
  struct real_avl_tree *node;
  struct avl_tree_traverse *next;
};  

/* to be used as an interface */

struct avl_tree *avl_create(avl_compare *compare);
int avl_insert(struct avl_tree *, void *value);
int avl_delete(struct avl_tree *tree, void *value);
void *avl_search(struct avl_tree *, void *value); 
int avl_count(struct avl_tree *tree);
void avl_delete_all(struct avl_tree *tree, avl_clear *del);

/* traversal functions */
struct avl_tree_traverse *init_traverse_tree(struct avl_tree *tree);
void *avl_traverse(struct avl_tree_traverse **t);
	/* note, a change to static depth will make this unnecessary */
void avl_traverse_free(struct avl_tree_traverse *t);

/* to be used internally */

          /* insertion */
int avl_node_insert(avl_compare *compare, struct real_avl_tree **node,
		void *value, int *count,struct avl_tree *main);

         /* rotation & insertion balance */
int avl_node_balance_left(struct real_avl_tree **node);
int avl_node_balance_right(struct real_avl_tree **node);
void avl_rr(struct real_avl_tree **node);
void avl_rl(struct real_avl_tree **node);

	/* deletion */ 
int avl_node_delete(avl_compare *compare, struct real_avl_tree **node,
	void *value, int *count,struct avl_tree *tree);
void avl_delete_all_node(struct real_avl_tree *node, avl_clear *del);

	/* rotation and deletion balance */
int avl_b_rl(struct real_avl_tree **node);
int avl_b_rr(struct real_avl_tree **node);
int avl_rightmost(struct real_avl_tree *target, struct real_avl_tree **n,
	int *retval);
int avl_leftmost(struct real_avl_tree *target, struct real_avl_tree **n,
	int *retval);


	/* searching */
void *avl_node_search(avl_compare *compare, struct real_avl_tree * node,
                void *value);

	/* traversal */
void init_traverse_node(struct real_avl_tree *node,
                struct avl_tree_traverse **t);

/* example comparison & deletion functions */
void generic_delete(void *simple_data);
int compare_numbers(void *left, void *right);

/* test function, inorder traversal that does nothing */
void avl_traverse_all(struct avl_tree *t);

